// The sample box model.  You should build a file
// very similar to this for when you make your model in
// order to plug in to the animator project.

#pragma warning (disable : 4305)
#pragma warning (disable : 4244)
#pragma warning(disable : 4786)
#pragma warning (disable : 4312)

#include "modelerview.h"
#include "modelerapp.h"
#include "modelerdraw.h"
#include <FL/gl.h>
#include <gl\glaux.h>
#include <math.h>


typedef struct {
	float xyz[3];
} vertex;
typedef struct {
	float uv[2];
} uv;
typedef struct {
	int indicies[3];
} poly;

typedef struct {
	vertex *verticies;
	int vertex_count;
	bool has_vertex_buffer;
	
	
	poly *polygons;
	int polygon_count;
	bool has_polygon_buffer;
	
	vertex *normals;
	int normal_count;
	bool has_normal_buffer;

	poly *polynormals;
	int polynormal_count;
	bool has_polynormal_buffer;
	
	int *material_indicies;
	float *materials;
	int material_count;
	bool has_material_buffer;

	
	


	} mesh;
mesh torso;
mesh head;
//GLuint	texture[1];

// This is a list of the controls for the RobotArm
// We'll use these constants to access the values 
// of the controls from the user interface.


enum StayPuftModelControls
{ 
	XPOS, YPOS, ZPOS, SPINX, SPINY, SPINZ, /*SCALE,*/ LEAN, TWIST, LOOKUP, LOOKSIDE, HEADSPIN, HEADNOD,
	R_HIP_ROTATE, R_KNEE_ROTATE, R_HIP_PIVOT, R_FOOT_PIVOT, R_FOOT_ROTATE,
	L_HIP_ROTATE, L_KNEE_ROTATE, L_HIP_PIVOT, L_FOOT_PIVOT, L_FOOT_ROTATE,
	R_SHOULDER_ROTATE, R_SHOULDER_WAVE, R_ELBOW_ROTATE, R_WRIST_ROTATE, R_WRIST_WAVE, R_WRIST_HINGE, R_HAND_GRAB,
	L_SHOULDER_ROTATE, L_SHOULDER_WAVE, L_ELBOW_ROTATE, L_WRIST_ROTATE, L_WRIST_WAVE, L_WRIST_HINGE, L_HAND_GRAB,
	NUMCONTROLS,
};

// To make a BoxModel, we inherit off of ModelerView
class StayPuftModel : public ModelerView 
{
public:
    // Constructor for the model.  In your model, 
    // make sure you call the ModelerView constructor,
    // as done below.
    StayPuftModel(int x, int y, int w, int h, char *label) 
        : ModelerView(x,y,w,h,label) {}
    virtual void draw();
};

// We need to make a creator function, mostly because of
// nasty API stuff that we'd rather stay away from.
ModelerView* createStayPuftModel(int x, int y, int w, int h, char *label)
{ 
    return new StayPuftModel(x,y,w,h,label); 
}

// We'll be getting the instance of the application a lot; 
// might as well have it as a macro.

#define VAL(x) (ModelerApplication::Instance()->GetControlValue(x))

// We are going to override (is that the right word?) the draw()
// method of ModelerView to draw out RobotArm


void drawSemiCircle(double r, double width, double startTheta, double curvatureR){
	setDiffuseColor(1,1,1);

	double r1 = r - width/2.0;
	double r2 = r + width/2.0;
	int theta;
	double theta1;
	double theta2;
	double z1, z2, z3, z4;
	double x1, x2, x3, x4;

	for(theta=startTheta; theta<=startTheta+179; theta=theta+4){
		theta1 = theta*3.14159265/180.0;
		theta2 = (theta+4)*3.141592654/180.0;
		x1 = r1*cos(theta1);
		x2 = r2*cos(theta1);
		x3 = r2*cos(theta2);
		x4 = r1*cos(theta2);

		z1 = -1*(curvatureR - curvatureR*curvatureR/sqrt(curvatureR*curvatureR + x1*x1));
		z2 = -1*(curvatureR - curvatureR*curvatureR/sqrt(curvatureR*curvatureR + x2*x2));
		z3 = -1*(curvatureR - curvatureR*curvatureR/sqrt(curvatureR*curvatureR + x3*x3));
		z4 = -1*(curvatureR - curvatureR*curvatureR/sqrt(curvatureR*curvatureR + x4*x4));

		x1 = x1*curvatureR/sqrt(curvatureR*curvatureR + x1*x1);
		x2 = x2*curvatureR/sqrt(curvatureR*curvatureR + x2*x2);
		x3 = x3*curvatureR/sqrt(curvatureR*curvatureR + x3*x3);
		x4 = x4*curvatureR/sqrt(curvatureR*curvatureR + x4*x4);

		glBegin(GL_POLYGON);
			glNormal3d(x1, r1*sin(theta1), z1+curvatureR);
			glVertex3d(x1, r1*sin(theta1), z1);
			glNormal3d(x2, r2*sin(theta1), z2+curvatureR);
			glVertex3d(x2, r2*sin(theta1), z2);
			glNormal3d(x3, r2*sin(theta2), z3+curvatureR);
			glVertex3d(x3, r2*sin(theta2), z3);
			glNormal3d(x4, r1*sin(theta2), z4+curvatureR);
			glVertex3d(x4, r1*sin(theta2), z4);
		glEnd();
	}
}

void drawCurve(double height, double width, double rotation, double curvatureR){
	setDiffuseColor(1,1,1);

	double x, y;
	double z1, z2, z3, z4;
	double x1, x2, x3, x4;

	double Yslices = height/15.0;
	double Xslices = width/15.0;
	double rot = rotation*3.14159265/180.0;
	double factor;
	if (rot<0)
		factor=-1;
	else factor=1;

	for(y=0; y<height; y=y+Yslices){
		for(x=0; x<width; x=x+Xslices){

		x1 = x + y*sin(rot);
		x2 = x + (y+Yslices)*sin(rot);
		x3 = x + Xslices + (y+Yslices)*sin(rot);
		x4 = x + Xslices + y*sin(rot);

		z1 = -1*factor*(curvatureR - curvatureR*curvatureR/sqrt(curvatureR*curvatureR + x1*x1));
		z2 = -1*factor*(curvatureR - curvatureR*curvatureR/sqrt(curvatureR*curvatureR + x2*x2));
		z3 = -1*factor*(curvatureR - curvatureR*curvatureR/sqrt(curvatureR*curvatureR + x3*x3));
		z4 = -1*factor*(curvatureR - curvatureR*curvatureR/sqrt(curvatureR*curvatureR + x4*x4));

		x1 = x1*curvatureR/sqrt(curvatureR*curvatureR + x1*x1);
		x2 = x2*curvatureR/sqrt(curvatureR*curvatureR + x2*x2);
		x3 = x3*curvatureR/sqrt(curvatureR*curvatureR + x3*x3);
		x4 = x4*curvatureR/sqrt(curvatureR*curvatureR + x4*x4);

		glBegin(GL_POLYGON);
			glNormal3d(x1, 0, z1+curvatureR);
			glVertex3d(x1, y, z1);
			glNormal3d(x2, Yslices, z2+curvatureR);
			glVertex3d(x2, y+Yslices, z2);
			glNormal3d(x3, Yslices, z3+curvatureR);
			glVertex3d(x3, y+Yslices, z3);
			glNormal3d(x4, 0, z4+curvatureR);
			glVertex3d(x4, y, z4);
		glEnd();
		}
	}
}


void drawMarshMallow(double h, double r1, double r2){

	double z;
	int theta;
	float theta1; 
	float theta2;
	float tempR1;
	float tempR2;
	float puff = 1000000.0;
	float factor = log((h/2.0)*puff);
	double slices = h/30.0;

	for (z=0.000001; z<h; z=z+slices){
		for (theta=0; theta<=359; theta=theta+10){
			theta1 = theta*3.14159265/180.0;
			theta2 = (theta+10)*3.14159265/180.0;
			tempR1 = (r1*(1-z/h)+r2*z/h)*log(((abs(abs(z - h/2.0) - h/2.0)))*puff)/factor;
			tempR2 = (r1*(1-(z+slices)/h)+r2*(z+slices)/h)*log(((abs(abs(z+slices - h/2.0) - h/2.0)))*puff)/factor;

			glBegin(GL_POLYGON);
				//glNormal3d( b*f - c*e, c*d - a*f, a*e - b*d );
				glNormal3d(tempR1*cos(theta1), tempR1*sin(theta1), z - h/2.0);
				glVertex3f(tempR1*cos(theta1), tempR1*sin(theta1), z);

				if(z+slices<h){
					glNormal3d(tempR2*cos(theta1), tempR2*sin(theta1), z+slices - h/2.0);
					glVertex3f(tempR2*cos(theta1), tempR2*sin(theta1), z+slices);
				}else{
					glNormal3d(0,0,1);
					glVertex3f(0,0,h);
				}
				
				if(z+slices<h){
					glNormal3d(tempR2*cos(theta2), tempR2*sin(theta2), z+slices - h/2.0);
					glVertex3f(tempR2*cos(theta2), tempR2*sin(theta2), z+slices);
				}else{
					glNormal3d(0,0,1);
					glVertex3f(0,0,h);
				}
				glNormal3d(tempR1*cos(theta2), tempR1*sin(theta2), z - h/2.0);
				glVertex3f(tempR1*cos(theta2), tempR1*sin(theta2), z);
			glEnd();
		}
	}
}

void DrawMesh(mesh *target, boolean invertN) {
	//glColor4f(1.0f,1.0f,1.0f,1.0f);
	setDiffuseColor(1,1,1);

 	glBegin(GL_TRIANGLES);
 		for (int i = 0; i < target->polygon_count; i++) {
			int n = 1;
			if (invertN) n = -1;
			if (target->has_normal_buffer) {
				if (target->has_material_buffer) {
					setDiffuseColor(target->materials[target->material_indicies[i]*3],target->materials[target->material_indicies[i]*3 + 1],target->materials[target->material_indicies[i]*3 + 2]);
				}
				glNormal3f(	n*target->normals[target->polynormals[i].indicies[0]].xyz[0],
							n*target->normals[target->polynormals[i].indicies[0]].xyz[1],
							n*target->normals[target->polynormals[i].indicies[0]].xyz[2]);
				glVertex3f(	target->verticies[target->polygons[i].indicies[0]].xyz[0],
							target->verticies[target->polygons[i].indicies[0]].xyz[1],
							target->verticies[target->polygons[i].indicies[0]].xyz[2]);


				glNormal3f(	n*target->normals[target->polynormals[i].indicies[1]].xyz[0],
							n*target->normals[target->polynormals[i].indicies[1]].xyz[1],
							n*target->normals[target->polynormals[i].indicies[1]].xyz[2]);
				glVertex3f(	target->verticies[target->polygons[i].indicies[1]].xyz[0],
							target->verticies[target->polygons[i].indicies[1]].xyz[1],
							target->verticies[target->polygons[i].indicies[1]].xyz[2]);


				glNormal3f(	n*target->normals[target->polynormals[i].indicies[2]].xyz[0],
							n*target->normals[target->polynormals[i].indicies[2]].xyz[1],
							n*target->normals[target->polynormals[i].indicies[2]].xyz[2]);
				glVertex3f(	target->verticies[target->polygons[i].indicies[2]].xyz[0],
							target->verticies[target->polygons[i].indicies[2]].xyz[1],
							target->verticies[target->polygons[i].indicies[2]].xyz[2]);
			} else {
			drawTriangle(
				target->verticies[target->polygons[i].indicies[0]].xyz[0],
					target->verticies[target->polygons[i].indicies[0]].xyz[1],
					target->verticies[target->polygons[i].indicies[0]].xyz[2],

				target->verticies[target->polygons[i].indicies[1]].xyz[0],
					target->verticies[target->polygons[i].indicies[1]].xyz[1],
					target->verticies[target->polygons[i].indicies[1]].xyz[2],

				target->verticies[target->polygons[i].indicies[2]].xyz[0],
					target->verticies[target->polygons[i].indicies[2]].xyz[1],
					target->verticies[target->polygons[i].indicies[2]].xyz[2]);
			}
			
 		}

 	glEnd();
	//glBegin(GL_POINTS);
	//	for (int i = 0; i < target->vertex_count; i++) {
	//		
	//		glVertex3f(target->verticies[i].xyz[0],target->verticies[i].xyz[1],target->verticies[i].xyz[2]);
	//	}
	//glEnd();
}

void drawRightArm()
{
	glScalef(1.1,1.1,1.1);
	setDiffuseColor(1,1,1);
	//shoulder
	glRotatef(5, 0, 0, 1.0);
	glRotatef(VAL(R_SHOULDER_WAVE)/3.0,0,0,1);
	glRotatef(VAL(R_SHOULDER_ROTATE), 1, 0, 0);
	glPushMatrix();
		glScalef(0.6,1,.75);
		drawSphere(0.45);
	glPopMatrix();

	//forearm
	glTranslated(0.1, .02, 0);
	glRotatef(2.0*VAL(R_SHOULDER_WAVE)/3.0,0,0,1);
	glRotatef(VAL(R_ELBOW_ROTATE), 0, 1, 0);
	glTranslated(.09, 0, 0);
	glRotatef(90, 0, 1, 0);
	glScalef(1,1,1.15);
	glPushMatrix();
		glScalef(.75, 1, 1);
		drawMarshMallow(.49, .301, .14);
	glPopMatrix();

	//wrist
	glTranslated(0, 0, .45);
	glRotatef(VAL(R_WRIST_HINGE), 0, 1, 0);
	glRotatef(VAL(R_WRIST_WAVE), 1, 0, 0);
	glRotatef(-VAL(R_SHOULDER_WAVE),1,0,0);
	glTranslated(0, 0, .07);
	glRotatef(VAL(R_WRIST_ROTATE), 0, 0, 1);
	glPushMatrix();
		glScalef(.75, 1, .6);
		drawSphere(0.1);
	glPopMatrix();

	//hand
	glTranslated(0, 0, .1);
	glRotatef(15, 0, 1, 0);
	glPushMatrix();
		glScalef(.8, 1, .8);
		glTranslated(0, -.05, 0);
		drawSphere(.08);
	glPopMatrix();
	glPushMatrix();
		glTranslated(0, .05, 0);
		glPushMatrix();
			glScalef(.8, 1, .8);
			drawSphere(.08);
		glPopMatrix();
		glRotated(-80, 1, 0, 0);
		glTranslated(0, 0, .06);
		glRotatef(VAL(R_HAND_GRAB)/2, 0, 1, 0);
		drawMarshMallow(.05, .04, .04);
		glTranslated(0,0,.052);
		glRotated(VAL(R_HAND_GRAB)/1.1, 0, 1, 0);
		drawMarshMallow(.05, .04, .04);
	glPopMatrix();

	glTranslated(0, 0, .04);
	glRotatef(VAL(R_HAND_GRAB)/2, 0, 1, 0);
	glTranslated(0, 0, .03);
	glPushMatrix();
		glScalef(.7, 1, .5);
		drawSphere(.1);
	glPopMatrix();
	glPushMatrix();
		glTranslated(0,.07,.03);
		glRotated(-15, 1, 0, 0);
		glRotatef(VAL(R_HAND_GRAB)/2, 0, 1, 0);
		drawMarshMallow(.06, .04, .04);
		glTranslated(0,0,.062);
		glRotated(VAL(R_HAND_GRAB)/1.1, 0, 1, 0);
		drawMarshMallow(.06, .04, .04);
	glPopMatrix();
	glPushMatrix();
		glTranslated(0,0,.05);
		//glRotated(-15, 1, 0, 0);
		glRotatef(VAL(R_HAND_GRAB)/2, 0, 1, 0);
		drawMarshMallow(.06, .04, .04);
		glTranslated(0,0,.062);
		glRotated(VAL(R_HAND_GRAB)/1.1, 0, 1, 0);
		drawMarshMallow(.06, .04, .04);
	glPopMatrix();
	glPushMatrix();
		glTranslated(0,-.07,.03);
		glRotated(15, 1, 0, 0);
		glRotatef(VAL(R_HAND_GRAB)/2, 0, 1, 0);
		drawMarshMallow(.06, .04, .04);
		glTranslated(0,0,.062);
		glRotated(VAL(R_HAND_GRAB)/1.1, 0, 1, 0);
		drawMarshMallow(.06, .04, .04);
	glPopMatrix();
}

void drawLeftArm()
{
	setDiffuseColor(1,1,1);
	glScalef(-1,1,1);
		
	glScalef(1.1,1.1,1.1);
	setDiffuseColor(1,1,1);
	//shoulder
	glRotatef(5, 0, 0, 1.0);
	glRotatef(VAL(L_SHOULDER_WAVE)/3.0, 0, 0, 1);
	glRotatef(VAL(L_SHOULDER_ROTATE), 1, 0, 0);
	glPushMatrix();
		glScalef(0.6,1,.75);
		drawSphere(0.45);
	glPopMatrix();

	//forearm
	glTranslated(0.1, .02, 0);
	glRotatef(2.0*VAL(L_SHOULDER_WAVE)/3.0, 0,0,1);
	glRotatef(VAL(L_ELBOW_ROTATE), 0, 1, 0);
	glTranslated(.09, 0, 0);
	glRotatef(90, 0, 1, 0);
	glScalef(1,1,1.15);
	glPushMatrix();
		glScalef(.75, 1, 1);
		drawMarshMallow(.49, .301, .14);
	glPopMatrix();

	//wrist
	glTranslated(0, 0, .45);
	glRotatef(VAL(L_WRIST_HINGE), 0, 1, 0);
	glRotatef(VAL(L_WRIST_WAVE), 1, 0, 0);
	glRotatef(-VAL(L_SHOULDER_WAVE),1,0,0);
	glTranslated(0, 0, .07);
	glRotatef(VAL(L_WRIST_ROTATE), 0, 0, 1);
	glPushMatrix();
		glScalef(.75, 1, .6);
		drawSphere(0.1);
	glPopMatrix();

	//hand
	glTranslated(0, 0, .1);
	glRotatef(15, 0, 1, 0);
	glPushMatrix();
		glScalef(.8, 1, .8);
		glTranslated(0, -.05, 0);
		drawSphere(.08);
	glPopMatrix();
	glPushMatrix();
		glTranslated(0, .05, 0);
		glPushMatrix();
			glScalef(.8, 1, .8);
			drawSphere(.08);
		glPopMatrix();
		glRotated(-80, 1, 0, 0);
		glTranslated(0, 0, .06);
		glRotatef(VAL(L_HAND_GRAB)/2, 0, 1, 0);
		drawMarshMallow(.05, .04, .04);
		glTranslated(0,0,.052);
		glRotated(VAL(L_HAND_GRAB)/1.1, 0, 1, 0);
		drawMarshMallow(.05, .04, .04);
	glPopMatrix();

	glTranslated(0, 0, .04);
	glRotatef(VAL(L_HAND_GRAB)/2, 0, 1, 0);
	glTranslated(0, 0, .03);
	glPushMatrix();
		glScalef(.7, 1, .5);
		drawSphere(.1);
	glPopMatrix();
	glPushMatrix();
		glTranslated(0,.07,.03);
		glRotated(-15, 1, 0, 0);
		glRotatef(VAL(L_HAND_GRAB)/2, 0, 1, 0);
		drawMarshMallow(.06, .04, .04);
		glTranslated(0,0,.062);
		glRotated(VAL(L_HAND_GRAB)/1.1, 0, 1, 0);
		drawMarshMallow(.06, .04, .04);
	glPopMatrix();
	glPushMatrix();
		glTranslated(0,0,.05);
		//glRotated(-15, 1, 0, 0);
		glRotatef(VAL(L_HAND_GRAB)/2, 0, 1, 0);
		drawMarshMallow(.06, .04, .04);
		glTranslated(0,0,.062);
		glRotated(VAL(L_HAND_GRAB)/1.1, 0, 1, 0);
		drawMarshMallow(.06, .04, .04);
	glPopMatrix();
	glPushMatrix();
		glTranslated(0,-.07,.03);
		glRotated(15, 1, 0, 0);
		glRotatef(VAL(L_HAND_GRAB)/2, 0, 1, 0);
		drawMarshMallow(.06, .04, .04);
		glTranslated(0,0,.062);
		glRotated(VAL(L_HAND_GRAB)/1.1, 0, 1, 0);
		drawMarshMallow(.06, .04, .04);
	glPopMatrix();
}


void drawRightLeg() {
	setDiffuseColor(1,1,1);
	glRotatef(VAL(R_HIP_PIVOT), 0,1,0);
	glTranslated(0, 0, -0.1);
	glRotatef(VAL(R_HIP_ROTATE), 1,0,0);
	glTranslated(0.0, -0.45, 0);
	glPushMatrix();
		glRotatef(15,0,0,1);
		glScalef(1,1.35,1.0);
		drawSphere(.5);
	glPopMatrix();
	glTranslated(0, -0.4, 0);
	glRotatef(VAL(R_KNEE_ROTATE),1,0,0);
	glTranslated(0, -0.3, 0);
	glPushMatrix();
		glRotatef(-10,0,0,1);
		glScalef(.75,1.0,.75);
		drawSphere(0.5); //shin
	glPopMatrix();
	glTranslated(0, -0.4, 0);

	glPushMatrix();	
		glScalef(1,0.7,1.0);
		drawSphere(0.3);
	glPopMatrix();
	glPushMatrix();	
		glRotatef(90,1,0,0);
		drawCylinder(0.2,0.3,0.3);
	glPopMatrix();
	glRotatef(VAL(R_FOOT_PIVOT),0,1,0);
	
	glTranslated(0, 0, -0.2);
	glRotatef(VAL(R_FOOT_ROTATE),1,0,0);
	glTranslated(0, 0, -0.2);
	glPushMatrix();	
		glScalef(1,0.7,1.0);
		drawSphere(0.35);
	glPopMatrix();
	glPushMatrix();	
		glRotatef(90,1,0,0);
		drawCylinder(0.2,0.35,0.35);
	glPopMatrix();
}
void drawLeftLeg() {
	setDiffuseColor(1,1,1);
	glScalef(-1,1,1);
	glRotatef(VAL(L_HIP_PIVOT), 0,1,0);
	glTranslated(0, 0, -0.1);
	glRotatef(VAL(L_HIP_ROTATE), 1,0,0);
	glTranslated(0.0, -0.45, 0);
	glPushMatrix();
		glRotatef(15,0,0,1);
		glScalef(1,1.35,1.0);
		drawSphere(.5);
	glPopMatrix();
	glTranslated(0, -0.4, 0);
	glRotatef(VAL(L_KNEE_ROTATE),1,0,0);
	glTranslated(0, -0.3, 0);
	glPushMatrix();
		glRotatef(-10,0,0,1);
		glScalef(.75,1.0,.75);
		drawSphere(0.5); //shin
	glPopMatrix();
	glTranslated(0, -0.4, 0);

	glPushMatrix();	
		glScalef(1,0.7,1.0);
		drawSphere(0.3);
	glPopMatrix();
	glPushMatrix();	
		glRotatef(90,1,0,0);
		drawCylinder(0.2,0.3,0.3);
	glPopMatrix();

	glRotatef(VAL(L_FOOT_PIVOT),0,1,0);
	glTranslated(0, 0, -0.2);
	glRotatef(VAL(L_FOOT_ROTATE),1,0,0);
	glTranslated(0, 0, -0.2);
	glPushMatrix();	
		glScalef(1,0.7,1.0);
		drawSphere(0.35);
	glPopMatrix();
	glPushMatrix();	
		glRotatef(90,1,0,0);
		drawCylinder(0.2,0.35,0.35);
	glPopMatrix();
	
}
void drawHead() {
	//drawSphere(0.5);
	glRotatef(VAL(HEADNOD),1,0,0);
	glRotatef(VAL(HEADSPIN),0,1,0);
	DrawMesh(&head, true);
	
	glPushMatrix();
		glTranslatef(0,0.55,-0.22);
		glPushMatrix();
			glTranslatef(0.19,0,0);
			
			setDiffuseColor(1.0f,1.0f,1.0f);
			drawSphere(0.12);
			glRotatef(VAL(LOOKUP),1,0,0);
			glRotatef(VAL(LOOKSIDE),0,1,0);
			glTranslatef(0.0,0,-0.07);
			setDiffuseColor(0.0f,0.0f,0.0f);
			drawSphere(0.06);
		glPopMatrix();
		glPushMatrix();
			glTranslatef(-0.19,0,0);
			setDiffuseColor(1.0f,1.0f,1.0f);
			drawSphere(0.12);
			glRotatef(VAL(LOOKUP),1,0,0);
			glRotatef(VAL(LOOKSIDE),0,1,0);
			glTranslatef(0.0,0,-0.07);
			setDiffuseColor(0.0f,0.0f,0.0f);
			drawSphere(0.06);
			
			setDiffuseColor(0.8f,0.8f,0.8f);
		glPopMatrix();
	glPopMatrix();
	glPushMatrix();
		glTranslatef(0.23,.93,0);
		
		glRotatef(-25,0,0,1);
		glTranslatef(0,.01,0);
		glPushMatrix();
			glRotatef(90,1,0,0);

			setDiffuseColor(0.0f,0.0f,0.627451f);
			drawCylinder(0.2,0.25,0.25); //hat
			setDiffuseColor(1,1,1);
			glRotatef(-90, 1, 0, 0);
			glRotatef(180,0,1,0);
			glRotatef(10,0,1,0);
			glPushMatrix();
				//"S"
				glTranslatef(0, -.06, .2501);
				drawSemiCircle(.019, .01, 90, .2501);
				glTranslatef(0,-.038,0);
				drawSemiCircle(.019, .01, 270, .2501);
			glPopMatrix();
			glRotatef(10.2, 0, 1, 0);
			glPushMatrix();
				//"T"
				glTranslatef(0,-.118,.2501);
				drawCurve(.07, .01, 0, .2501);
				glTranslatef(-.02,.06,0);
				drawCurve(.01, .05, 5, .2501);
			glPopMatrix();
			glRotatef(7,0,1,0);
			glPushMatrix();
				//"A"
				glTranslatef(0,-.12,.2501);
				drawCurve(.075,.01,15,.2501);
				glTranslatef(0,0,-0.2501);
				glPushMatrix();
					glRotatef(-1.8,0,1,0);
					glTranslatef(0,0,0.2501);
					glTranslatef(.02,.03,0);
					drawCurve(.01,.022,0,.2501);
				glPopMatrix();
				glRotatef(9,0,1,0);
				glTranslatef(0,0,0.2501);
				drawCurve(.075,.01,-15,.2501);
				//glTranslatef(-.03, .03, 0);
				//drawCurve(.01,.028,0,.2501);
			glPopMatrix();
			glRotatef(16,0,1,0);
			glPushMatrix();
				//"Y"
				glTranslatef(0,-.08,.2501);
				drawCurve(.04,.01,-35,.2501);
				drawCurve(.038,.01,35,.2501);
				glTranslatef(0,-.039,0);
				drawCurve(.040,.01,0,.2501);
			glPopMatrix();
			glRotatef(25,0,1,0);
			glPushMatrix();
				//"P"
				glTranslatef(0,-.065,.2501);
				drawSemiCircle(.018, .01, -90, .2501);
				glTranslatef(-.009, -.052, 0);
				drawCurve(.075,.01,0,.2501);
			glPopMatrix();
			glRotatef(8.5,0,1,0);
			glPushMatrix();
				//"U"
				glTranslatef(0,-.09,.2501);
				drawCurve(.045,.01,0,.2501);
				glTranslatef(0,.1,-.2501);
				glRotatef(5.5,0,1,0);
				glTranslatef(0,-.1,.2501);
				drawSemiCircle(.019,.01,180,.2501);
				glTranslatef(0,.1,-.2501);
				glRotatef(3.1,0,1,0);
				glTranslatef(0,-.102,.2501);
				drawCurve(.045,.01,0,.2501);
			glPopMatrix();
			glRotatef(15,0,1,0);
			glPushMatrix();
				//"F"
				glTranslatef(0,-.114,.2501);
				drawCurve(.07,.01,0,.2501);
				glTranslatef(0,.032,0);
				drawCurve(.01,.03,0,.2501);
				glTranslatef(0,.03,0);
				drawCurve(.01,.035,0,.2501);
			glPopMatrix();
			glRotatef(15,0,1,0);
			glPushMatrix();
				//"T"
				glTranslatef(0,-.112,.2501);
				drawCurve(.07, .01, 0, .2501);
				glTranslatef(-.02,.06,0.001);
				drawCurve(.01, .05, 0, .2501);
			glPopMatrix();


		glPopMatrix();
		glTranslatef(0,0.1,0);
		glScalef(1,0.4,1);
		drawSphere(0.4);
		glTranslatef(0,.4,0);
		setDiffuseColor(.2,.2,.8);
		drawSphere(.1);
	glPopMatrix();
}

void drawCopCar(){

	glScalef(1,1,1.2);
	//wheels
	glPushMatrix();
		setDiffuseColor(.2,.2,.2);
		glTranslated(.8,0,-.01);
		drawCylinder(.2,.4,.4);
		glTranslated(0,0,-.01);
		setDiffuseColor(1,1,1);
		drawCylinder(.02,.15,.15);
	glPopMatrix();
	glPushMatrix();
		setDiffuseColor(.2,.2,.2);
		glTranslated(.8,0,1.51);
		drawCylinder(.2,.4,.4);
		glTranslated(0,0,.21);
		setDiffuseColor(1,1,1);
		drawCylinder(.01,.15,.15);
	glPopMatrix();
	glPushMatrix();
		setDiffuseColor(.2,.2,.2);
		glTranslated(3.5,0,-.01);
		drawCylinder(.2,.4,.4);
		glTranslated(0,0,-.01);
		setDiffuseColor(1,1,1);
		drawCylinder(.02,.15,.15);
	glPopMatrix();
	glPushMatrix();
		setDiffuseColor(.2,.2,.2);
		glTranslated(3.5,0,1.51);
		drawCylinder(.2,.4,.4);
		glTranslated(0,0,.21);
		setDiffuseColor(1,1,1);
		drawCylinder(.01,.15,.15);
	glPopMatrix();

	//base
	setDiffuseColor(0.1, 0.1, 0.1);
	drawBox(1, .8, 1.7);
	glPushMatrix();
		glTranslated(-.02,.6,.2);
		setDiffuseColor(1,.1,.1);
		drawBox(.1,.13,.4);
		glTranslated(0,-.4,.4);
		setDiffuseColor(1,1,1);
		drawBox(.1,.2,.5);
		glTranslated(0,.4,.5);
		setDiffuseColor(1,.1,.1);
		drawBox(.1,.13,.4);
	glPopMatrix();
	glTranslated(1,0,0);
	glPushMatrix();
		setDiffuseColor(1, 1, 1);
		drawBox(1, .8, 1.7);
		setDiffuseColor(0.1, 0.1, 0.1);
		glTranslated(1, 0, 0);
		drawBox(.1, .8, 1.7);
		glTranslated(.1, 0, 0);
		setDiffuseColor(1, 1, 1);
		drawBox(1, .8, 1.7);
		glTranslated(1,0,0);
		setDiffuseColor(0.1, 0.1, 0.1);
		drawBox(1.2, .8, 1.7);
		glTranslated(1.2,.5,.3);
		setDiffuseColor(1,1,1);
		glScalef(.2,1,1);
		drawSphere(.15);
		glTranslated(0,0,1.1);
		drawSphere(.15);
	glPopMatrix();

	//back windows
	glTranslated(0,.8,0);
	drawBox(2.1, .05, 1.7);
	glTranslated(0,.05,0);
	setDiffuseColor(1, 1, 1);
	glBegin(GL_TRIANGLES);
		glVertex3f(.3, 0, 0);
		glVertex3f(0, 0, 0);
		glVertex3f(.3, .5, 0);
	glEnd();
	glBegin(GL_TRIANGLES);
		glVertex3f(.3, 0, 1.7);
		glVertex3f(0, 0, 1.7);
		glVertex3f(.3, .5, 1.7);
	glEnd();
	glBegin(GL_POLYGON);
		glVertex3f(0, 0, 0);
		glVertex3f(0, 0, 1.7);
		glVertex3f(.3, .5, 1.7);
		glVertex3f(.3, .5, 0);
	glEnd();

	glTranslated(.3, 0, 0);
	setDiffuseColor(.1,.1,.1);
	drawBox(.05,.5,1.7);
	glTranslated(.05,0,0);
	setDiffuseColor(1,1,1);
	drawBox(.65,.5,1.7);

	//roof
	glPushMatrix();
		setDiffuseColor(1,1,1);
		glTranslated(0,.5,0);
		drawBox(1.4,.05,1.7);
		glTranslated(.65, .2, 0);
		setDiffuseColor(1,.1,.1);
		drawCylinder(.7,.15,.15);
		glTranslated(0,0,.7);
		setDiffuseColor(1,1,1);
		drawCylinder(.3,.15,.15);
		glTranslated(0,0,.3);
		setDiffuseColor(.1,.1,1);
		drawCylinder(.7,.15,.15);
	glPopMatrix();

	//front windows
	setDiffuseColor(.1,.1,.1);
	glTranslated(.65,0,0);
	drawBox(.1,.5,1.7);
	setDiffuseColor(1,1,1);
	glTranslated(.1,0,0);
	drawBox(.65,.5,1.7);
	setDiffuseColor(.1,.1,.1);
	glTranslated(.65,0,0);
	drawBox(.05,.5,1.7);
	glTranslated(.05,0,0);
	setDiffuseColor(1,1,1);
	glBegin(GL_TRIANGLES);
		glVertex3f(.3, 0, 0);
		glVertex3f(0, 0, 0);
		glVertex3f(0, .5, 0);
	glEnd();
	glBegin(GL_TRIANGLES);
		glVertex3f(.3, 0, 1.7);
		glVertex3f(0, 0, 1.7);
		glVertex3f(0, .5, 1.7);
	glEnd();
	glBegin(GL_POLYGON);
		glVertex3f(0.3, 0, 0.05);
		glVertex3f(0.3, 0, 1.65);
		glVertex3f(0, .5, 1.65);
		glVertex3f(0, .5, 0.05);
	glEnd();
	setDiffuseColor(.1,.1,.1);
	glBegin(GL_POLYGON);
		glVertex3f(0.3, 0, 0);
		glVertex3f(0.3, 0, .05);
		glVertex3f(0, .5, .05);
		glVertex3f(0, .5, 0);
	glEnd();
	glBegin(GL_POLYGON);
		glVertex3f(0.3, 0, 1.65);
		glVertex3f(0.3, 0, 1.7);
		glVertex3f(0, .5, 1.7);
		glVertex3f(0, .5, 1.65);
	glEnd();
}

void drawTaxi(){

	glScalef(1,1,1.2);
	//wheels
	glPushMatrix();
		setDiffuseColor(.2,.2,.2);
		glTranslated(.8,0,-.01);
		drawCylinder(.2,.4,.4);
		glTranslated(0,0,-.01);
		setDiffuseColor(1,1,1);
		drawCylinder(.02,.15,.15);
	glPopMatrix();
	glPushMatrix();
		setDiffuseColor(.2,.2,.2);
		glTranslated(.8,0,1.51);
		drawCylinder(.2,.4,.4);
		glTranslated(0,0,.21);
		setDiffuseColor(1,1,1);
		drawCylinder(.01,.15,.15);
	glPopMatrix();
	glPushMatrix();
		setDiffuseColor(.2,.2,.2);
		glTranslated(3.5,0,-.01);
		drawCylinder(.2,.4,.4);
		glTranslated(0,0,-.01);
		setDiffuseColor(1,1,1);
		drawCylinder(.02,.15,.15);
	glPopMatrix();
	glPushMatrix();
		setDiffuseColor(.2,.2,.2);
		glTranslated(3.5,0,1.51);
		drawCylinder(.2,.4,.4);
		glTranslated(0,0,.21);
		setDiffuseColor(1,1,1);
		drawCylinder(.01,.15,.15);
	glPopMatrix();

	//base
	setDiffuseColor(1, 1, 0.1);
	drawBox(1, .8, 1.7);
	glPushMatrix();
		glTranslated(-.02,.6,.2);
		setDiffuseColor(1,.1,.1);
		drawBox(.1,.13,.4);
		glTranslated(0,-.4,.4);
		setDiffuseColor(1,1,1);
		drawBox(.1,.2,.5);
		glTranslated(0,.4,.5);
		setDiffuseColor(1,.1,.1);
		drawBox(.1,.13,.4);
	glPopMatrix();
	glTranslated(1,0,0);
	glPushMatrix();
		setDiffuseColor(1, 1, .1);
		drawBox(1, .8, 1.7);
		setDiffuseColor(0.1, 0.1, 0.1);
		glTranslated(1, 0, 0);
		drawBox(.1, .8, 1.7);
		glTranslated(.1, 0, 0);
		setDiffuseColor(1, 1, .1);
		drawBox(1, .8, 1.7);
		glTranslated(1,0,0);
		setDiffuseColor(1, 1, 0.1);
		drawBox(1.2, .8, 1.7);
		glTranslated(1.2,.5,.3);
		setDiffuseColor(1,1,1);
		glScalef(.2,1,1);
		drawSphere(.15);
		glTranslated(0,0,1.1);
		drawSphere(.15);
	glPopMatrix();

	//back windows
	glTranslated(0,.8,0);
	drawBox(2.1, .05, 1.7);
	glTranslated(0,.05,0);
	setDiffuseColor(1, 1, 1);
	glBegin(GL_TRIANGLES);
		glVertex3f(.3, 0, 0);
		glVertex3f(0, 0, 0);
		glVertex3f(.3, .5, 0);
	glEnd();
	glBegin(GL_TRIANGLES);
		glVertex3f(.3, 0, 1.7);
		glVertex3f(0, 0, 1.7);
		glVertex3f(.3, .5, 1.7);
	glEnd();
	glBegin(GL_POLYGON);
		glVertex3f(0, 0, 0);
		glVertex3f(0, 0, 1.7);
		glVertex3f(.3, .5, 1.7);
		glVertex3f(.3, .5, 0);
	glEnd();

	glTranslated(.3, 0, 0);
	setDiffuseColor(.1,.1,.1);
	drawBox(.05,.5,1.7);
	glTranslated(.05,0,0);
	setDiffuseColor(1,1,1);
	drawBox(.65,.5,1.7);

	//roof
	glPushMatrix();
		setDiffuseColor(1,1,.1);
		glTranslated(0,.5,0);
		drawBox(1.4,.05,1.7);
		glTranslated(.6, .05, .65);
		setDiffuseColor(1,1,1);
		drawBox(.1,.2,.4);
	glPopMatrix();

	//front windows
	setDiffuseColor(.1,.1,.1);
	glTranslated(.65,0,0);
	drawBox(.1,.5,1.7);
	setDiffuseColor(1,1,1);
	glTranslated(.1,0,0);
	drawBox(.65,.5,1.7);
	setDiffuseColor(.1,.1,.1);
	glTranslated(.65,0,0);
	drawBox(.05,.5,1.7);
	glTranslated(.05,0,0);
	setDiffuseColor(1,1,1);
	glBegin(GL_TRIANGLES);
		glVertex3f(.3, 0, 0);
		glVertex3f(0, 0, 0);
		glVertex3f(0, .5, 0);
	glEnd();
	glBegin(GL_TRIANGLES);
		glVertex3f(.3, 0, 1.7);
		glVertex3f(0, 0, 1.7);
		glVertex3f(0, .5, 1.7);
	glEnd();
	glBegin(GL_POLYGON);
		glVertex3f(0.3, 0, 0.05);
		glVertex3f(0.3, 0, 1.65);
		glVertex3f(0, .5, 1.65);
		glVertex3f(0, .5, 0.05);
	glEnd();
	setDiffuseColor(.1,.1,.1);
	glBegin(GL_POLYGON);
		glVertex3f(0.3, 0, 0);
		glVertex3f(0.3, 0, .05);
		glVertex3f(0, .5, .05);
		glVertex3f(0, .5, 0);
	glEnd();
	glBegin(GL_POLYGON);
		glVertex3f(0.3, 0, 1.65);
		glVertex3f(0.3, 0, 1.7);
		glVertex3f(0, .5, 1.7);
		glVertex3f(0, .5, 1.65);
	glEnd();
}

void drawRoad(){
	glPushMatrix();

	double z;
	setDiffuseColor(.8,.8,.2);
	glPushMatrix();
		glTranslatef(-.03,0,-5.3);
		for(z=0;z<9.7;z=z+.3){
			glTranslatef(0,0,.3);
			drawBox(.06,.002,.2);
		}
	glPopMatrix();
	setDiffuseColor(.1,.1,.1);
	glTranslatef(-1,0,-5);
	drawBox(2,.001,10);
	setDiffuseColor(.3,.3,.3);
	drawBox(.2,.01,10);
	glTranslatef(1.8,0,0);
	drawBox(.2,.01,10);

	glPopMatrix();
}


void drawBody() {
	glPushMatrix();
	glRotatef(VAL(SPINX),1,0,0);
	glRotatef(VAL(SPINY),0,1,0);
	glRotatef(VAL(SPINZ),0,0,1);
	glTranslated(VAL(XPOS), VAL(YPOS), VAL(ZPOS));
	//glScalef(VAL(SCALE), VAL(SCALE), VAL(SCALE));
		//glTranslated(0,2,0);

		glPushMatrix();
			
			glTranslated(-0.4, -0.2, 0);
			drawLeftLeg();
		glPopMatrix();

		glPushMatrix();
			
			glTranslated(0.4, -0.2, 0);
			drawRightLeg();
		glPopMatrix();

		glRotatef(VAL(TWIST), 0, 1, 0);
		glRotatef(VAL(LEAN), 1, 0, 0);

		glPushMatrix();
			
			DrawMesh(&torso,true);
		glPopMatrix();
		glPushMatrix();
			glTranslated(0,1.45,0);
			drawHead();
		glPopMatrix();
		glPushMatrix();
			glTranslated(0.85,1,0);		
			drawRightArm();
		glPopMatrix();
		glPushMatrix();
			glTranslated(-0.85,1,0);
			drawLeftArm();
		glPopMatrix();
		glPopMatrix();
	glPopMatrix();
}

double time = 0;
void StayPuftModel::draw()
{
    // This call takes care of a lot of the nasty projection 
    // matrix stuff.  Unless you want to fudge directly with the 
	// projection matrix, don't bother with this ...
    ModelerView::draw();
	if (ModelerApplication::Instance()->GetAnimating() ) {
		
		time += 0.1;
		
		ModelerApplication::Instance()->SetControlValue(R_SHOULDER_WAVE, 2.5 + 32.5*sin(time));
		ModelerApplication::Instance()->SetControlValue(L_SHOULDER_WAVE, 2.5 + 32.5*cos(time/2));
		ModelerApplication::Instance()->SetControlValue(R_FOOT_ROTATE, 20 - 20*pow(sin(time),2));
		ModelerApplication::Instance()->SetControlValue(LOOKSIDE,sin(time)*15);
		ModelerApplication::Instance()->SetControlValue(R_KNEE_ROTATE, -5 * (sin(time)+1) );
		ModelerApplication::Instance()->SetControlValue(R_HIP_ROTATE, 5 *(sin(time)+1) + 30 );
		
		
	}
	

	
	//glBegin(GL_LINES);
	//	glColor3f(1,1,1);
	//	glVertex3f(0,0,0);
	//	glVertex3f(5,0,0);
	//	glVertex3f(0,0,0);
	//	glVertex3f(0,5,0);
	//	glVertex3f(0,0,0);
	//	glVertex3f(0,0,5);
	//glEnd();
	// draw the road
	setAmbientColor(.1f,.1f,.1f);
	setDiffuseColor(0.8f,0.8f,0.8f);
	glTranslatef(0,-2,0);

	drawRoad();
	drawBody();

	glPushMatrix();
		glTranslated(-.2, .09, 1.5);
		glScalef(.17, .17, .17);
		glRotatef(230, 0, 1, 0);
		drawTaxi();
	glPopMatrix();
	glPushMatrix();
		glTranslated(.2, .09, -2.5);
		glRotatef(90,0,1,0);
		glScalef(.17,.17,.17);
		drawCopCar();
	glPopMatrix();
}



void LoadMeshFromFile(char *filename, mesh *DestinationMesh) {
	
	FILE *source_file = fopen(filename, "rt");
	
	//temp vars
	char readline[255];
	int temp;
	
	//vertexBuf
	fgets(readline, 255, source_file);
	sscanf(readline, " %i;", &temp);

	DestinationMesh->has_vertex_buffer = true;
	DestinationMesh->vertex_count = temp;
	DestinationMesh->verticies = (vertex*) malloc(sizeof(vertex) * DestinationMesh->vertex_count);
	
	for (int i = 0; i < DestinationMesh->vertex_count; i++) {
		fgets(readline, 255, source_file);
		
		sscanf( readline, "  %f;%f;%f;,", &(DestinationMesh->verticies[i].xyz[0]), &(DestinationMesh->verticies[i].xyz[1]), &(DestinationMesh->verticies[i].xyz[2]));
		//printf("v %f %f %f",DestinationMesh->verticies[i].xyz[0],DestinationMesh->verticies[i].xyz[1], DestinationMesh->verticies[i].xyz[2]);
		
	}
	
	//polyBuf
	fgets(readline, 255, source_file);
	sscanf(readline, " %i;", &temp);
	
	DestinationMesh->has_polygon_buffer = true;
	DestinationMesh->polygon_count = temp;
	DestinationMesh->polygons = (poly*) malloc(sizeof(poly) * DestinationMesh->polygon_count);
	
	for (int i = 0; i < DestinationMesh->polygon_count; i++) {
		fgets(readline, 255, source_file);
		sscanf( readline, "  3;%i,%i,%i;,", &(DestinationMesh->polygons[i].indicies[0]), &(DestinationMesh->polygons[i].indicies[1]), &(DestinationMesh->polygons[i].indicies[2]));
		//printf("vi %i %i %i", DestinationMesh->polygons[i].indicies[0], DestinationMesh->polygons[i].indicies[1], DestinationMesh->polygons[i].indicies[2]);
	}


	//materialBuffer

	fgets(readline, 255, source_file);
	sscanf(readline, " %i;", &temp);
	//printf("Material Count %i\n",temp);
	DestinationMesh->material_count = temp;
	

	fgets(readline, 255, source_file);
	sscanf(readline, " %i;", &temp);
	//printf("Material indicies %i\n",temp);
	DestinationMesh->has_material_buffer = true;
	if (temp >= 0) {
		
		DestinationMesh->materials = (float*) malloc(sizeof(float) * 3); //allocate matterial space
		DestinationMesh->material_indicies = (int*) malloc(sizeof(int) * DestinationMesh->polygon_count); //one for each poly
		
		for (int i = 0; i < DestinationMesh->polygon_count; i++) {
			fgets(readline, 255, source_file);
			sscanf( readline, "  %i,", &(DestinationMesh->material_indicies[i]));
			//printf("mi %i",DestinationMesh->material_indicies[i]);
		}
		for (int i = 0; i < DestinationMesh->material_count; i++) {
			fgets(readline, 255, source_file);
			sscanf( readline, "    %f;%f;%f;;", &(DestinationMesh->materials[i*3]), &(DestinationMesh->materials[i*3+1]), &(DestinationMesh->materials[i*3+2])  );\
			printf("m %f %f %f",DestinationMesh->materials[i*3],DestinationMesh->materials[i*3+1],DestinationMesh->materials[i*3+2]);
		}
	} else {
		printf("\nNo Material List Found\n");
		DestinationMesh->has_material_buffer = false;
	}


	//normal buffer
	fgets(readline, 255, source_file);
	sscanf(readline, " %i;", &temp);
	DestinationMesh->normal_count = temp;
	if (temp >= 0) {
		DestinationMesh->has_normal_buffer = true;
		//printf("Normal Count %i", DestinationMesh->normal_count);
		DestinationMesh->normals = (vertex*) malloc(sizeof(vertex) * DestinationMesh->normal_count);
		
		for (int i = 0; i < DestinationMesh->normal_count; i++) {
			fgets(readline, 255, source_file);
			//flip axis y-z cause model exporter breaks it
			sscanf( readline, "   %f;%f;%f;,", &(DestinationMesh->normals[i].xyz[0]),&(DestinationMesh->normals[i].xyz[1]),&(DestinationMesh->normals[i].xyz[2]));
			//printf("n %f %f %f", DestinationMesh->normals[i].xyz[0],DestinationMesh->normals[i].xyz[1],DestinationMesh->normals[i].xyz[2]);
		}
	} else {
		printf("\nNo No Normal List Found\n");
		DestinationMesh->has_normal_buffer = false;
	}



	//normal indicies
	fgets(readline, 255, source_file);
	sscanf(readline, "   %i;", &temp);
	DestinationMesh->polynormal_count = temp;
	if (temp >= 0) {
		DestinationMesh->has_polynormal_buffer = true;
		
		DestinationMesh->polynormals = (poly*) malloc(sizeof(poly) * DestinationMesh->polynormal_count);
		
		for (int i = 0; i < DestinationMesh->polynormal_count; i++) {
			fgets(readline, 255, source_file);
			sscanf( readline, "   3;%i,%i,%i;,", &(DestinationMesh->polynormals[i].indicies[0]), &(DestinationMesh->polynormals[i].indicies[1]), &(DestinationMesh->polynormals[i].indicies[2]));
			//printf("ni %i %i %i", DestinationMesh->polynormals[i].indicies[0], DestinationMesh->polynormals[i].indicies[1], DestinationMesh->polynormals[i].indicies[2]);
		}
	} else {
		printf("\nNo No Normal Indicies List Found\n");
		DestinationMesh->has_polynormal_buffer = false;
	}

	fclose(source_file);

	return;
}

int main()
{
	// Initialize the controls
	// Constructor is ModelerControl(name, minimumvalue, maximumvalue, 
	//								 stepsize, defaultvalue)
    // You will want to modify this to accommodate your model.

	

    ModelerControl controls[NUMCONTROLS];
	controls[XPOS]   = ModelerControl("X Position", -5, 5, 0.01f, 0);
	controls[YPOS]   = ModelerControl("Y Position", -5, 5, 0.01f, 1.9);
	controls[ZPOS]   = ModelerControl("Z Position", -5, 5, 0.01f, 0);
	
	controls[SPINX]   = ModelerControl("X Rotation", -180, 180, 5, 0);
	controls[SPINY]   = ModelerControl("Y Rotation", -180, 180, 5, 180);
	controls[SPINZ]   = ModelerControl("Z Rotation", -180, 180, 5, 0);

	//controls[SCALE] = ModelerControl("Scale", 0,3, 0.1, 1);

	controls[R_HIP_ROTATE] = ModelerControl("Right Hip Rotate", -30,54, 1, 30);
	controls[R_KNEE_ROTATE] = ModelerControl("Right Knee Rotate", -90,5, 1, 0);
	controls[R_HIP_PIVOT] = ModelerControl("Right Hip Pivot", -70,15, 1, 0);
	controls[R_FOOT_PIVOT] = ModelerControl("Right Foot Pivot", -60,30, 1, 0);
	controls[R_FOOT_ROTATE] = ModelerControl("Right Foot Rotate", -20,30, 1, -2);

	controls[LEAN] = ModelerControl("Lean", -45,25, 1, -12);
	controls[TWIST] = ModelerControl("Twist", -40,40, 1, 17);

	controls[L_HIP_ROTATE] = ModelerControl("Left Hip Rotate", -30,54, 1, -30);
	controls[L_KNEE_ROTATE] = ModelerControl("Left Knee Rotate", -90,5, 1, 0);
	controls[L_HIP_PIVOT] = ModelerControl("Left Hip Pivot", -70,15, 1, 0);
	controls[L_FOOT_PIVOT] = ModelerControl("Left Foot Pivot", -60,30, 1, 0);
	controls[L_FOOT_ROTATE] = ModelerControl("Left Foot Rotate", -20,30, 1, 30);

	controls[HEADSPIN] = ModelerControl("Turn Head", -45,45, 1, 0);
	controls[HEADNOD] = ModelerControl("Nod head", -30,20, 1, -6);

	controls[LOOKSIDE] = ModelerControl("Look Side", -15,15, 1, -9);
	controls[LOOKUP] = ModelerControl("Look Up", -15,25, 1, -9);

	controls[R_SHOULDER_ROTATE] = ModelerControl("Right Shoulder Rotate", -25,25, 1, -12);
	controls[R_SHOULDER_WAVE] = ModelerControl("Right Shoulder Wave", -20,35, 1, -20);
	controls[R_ELBOW_ROTATE] = ModelerControl("Right Elbow Bend", -10,55, 1, -10);
	controls[R_WRIST_ROTATE] = ModelerControl("Right Wrist Rotate", -35,35, 1, 0);
	controls[R_WRIST_WAVE] = ModelerControl("Right Wrist Wave", -25,20, 1, 0);
	controls[R_WRIST_HINGE] = ModelerControl("Right Wrist Hinge", -15,45, 1, -15);
	controls[R_HAND_GRAB] = ModelerControl("Right Hand Grab", 0,100, 1, 18);

	controls[L_SHOULDER_ROTATE] = ModelerControl("Left Shoulder Rotate", -25,25, 1, 0);
	controls[L_SHOULDER_WAVE] = ModelerControl("Left Shoulder Wave", -20,35, 1, 0);
	controls[L_ELBOW_ROTATE] = ModelerControl("Left Elbow Bend", -10,55, 1, 39);
	controls[L_WRIST_ROTATE] = ModelerControl("Left Wrist Rotate", -35,35, 1, 0);
	controls[L_WRIST_WAVE] = ModelerControl("Left Wrist Wave", -25,20, 1, 0);
	controls[L_WRIST_HINGE] = ModelerControl("Left Wrist Hinge", -15,45, 1, 0);
	controls[L_HAND_GRAB] = ModelerControl("Left Hand Grab", 0,100, 1, 29);

    // Initialize the modeler application with your model and the
    // appropriate array of controls.
    ModelerApplication::Instance()->Init(&createStayPuftModel, controls, NUMCONTROLS);

	LoadMeshFromFile("resources\\head2.x", &head);
	LoadMeshFromFile("resources\\lowBodyF.x", &torso);

	

	// make sure we give back the memory to older OSs that don't 
	// clear your memory pool after shutdown.
    int Result = ModelerApplication::Instance()->Run();
	delete ModelerApplication::Instance();
	return Result;
}



